import { APIGatewayProxyEvent, Context } from "aws-lambda";
import { Request } from "./types";
import { Static, TAny, TSchema } from "@sinclair/typebox";
declare type AnyRequest<TBody> = {
    pathParams: any;
    queryParams: any;
    body: TBody;
    response: (statusCode: number, body: any, handlers?: Record<string, string>) => Promise<Response<any, any>>;
};
export declare type RouteHandlerDefinition = {
    method: string;
    url: string;
    body?: TSchema;
    responses: Responses;
    handler: (req: AnyRequest<any>, apiParams: {
        originalEvent: APIGatewayProxyEvent;
        context: Context;
    }) => Promise<Response<any, any>>;
};
declare type HttpMethod<R, M extends HTTPMethod> = <A extends string, S extends StatusCode, B extends TSchema = never, Resp extends Responses = AnyType>(path: A, bodyOrResponses?: M extends HTTPRead ? Resp : B, responses?: M extends HTTPRead ? never : Resp) => (handler: (req: Request<A, Static<B>, Resp>, originalEvent: {
    originalEvent: APIGatewayProxyEvent;
    context: Context;
}) => Promise<Response<Resp, S>>) => Router<R>;
export declare type Router<R> = R & {
    get: HttpMethod<R, "get">;
    head: HttpMethod<R, "head">;
    options: HttpMethod<R, "options">;
    post: HttpMethod<R, "post">;
    put: HttpMethod<R, "put">;
    delete: HttpMethod<R, "delete">;
};
export declare type RouteHandlers = {
    handlers: readonly RouteHandlerDefinition[];
};
declare type HTTPRead = "get" | "options" | "head";
declare type HTTPWrite = "post" | "put" | "delete";
declare type HTTPMethod = HTTPRead | HTTPWrite;
export declare type Responses = Partial<{
    [k in StatusCode]: TSchema;
}>;
export declare type Response<R extends Responses, Status extends keyof R & StatusCode> = {
    statusCode: Status;
    headers?: {
        [header: string]: boolean | number | string;
    };
    body: Static<Status extends keyof R ? Static<R[Status]> : any>;
};
declare const StatusCodes: readonly [200, 201, 202, 203, 204, 205, 206, 207, 208, 226, 300, 301, 302, 303, 304, 305, 306, 307, 308, 400, 401, 402, 403, 404, 405, 406, 407, 408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 418, 420, 422, 423, 424, 425, 426, 428, 429, 431, 444, 449, 450, 451, 499, 500, 501, 502, 503, 504, 505, 506, 507, 508, 509, 510, 511, 598, 599];
export declare type StatusCode = typeof StatusCodes[number] & number;
declare type AnyType = Record<StatusCode, TAny>;
export declare const Router: (handlers?: readonly RouteHandlerDefinition[]) => Router<RouteHandlers>;
export {};
